@@ -1,42 +1,57 @@
 # -*- test-case-name: twisted.test.test_newcred -*-
 
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2003 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
+"""
+The point of integration of application and authentication.
+"""
+
 
 from twisted.internet import defer
 from twisted.internet.defer import maybeDeferred
 from twisted.python import failure, reflect, components
 from twisted.cred import error
+from zope import interface
 
 
 class IRealm(components.Interface):
-
+    """
+    The realm connects application-specific objects to the
+    authentication system.
+    """
     def requestAvatar(self, avatarId, mind, *interfaces):
         """Return avatar implementing one of the given interfaces.
 
-        Return a deferred which will fire a tuple of (interface,
-        avatarAspect, logout), or the tuple itself.  The interface
-        will be one of the interfaces passed in the 'interfaces'
-        argument.  The 'avatarAspect' will implement that interface.
-        The 'logout' object is a callable which will detach the mind
-        from the avatar.
+        @param avatarId: a string that identifies an avatar, as returned by
+            L{ICredentialsChecker.requestAvatarId<twisted.cred.checkers.ICredentialsChecker.requestAvatarId>}
+            (via a Deferred).  Alternatively, it may be
+            C{twisted.cred.checkers.ANONYMOUS}.
+        @param mind: usually None.  See the description of mind in
+            L{Portal.login}.
+        @param interfaces: the interface(s) the returned avatar should
+            implement, e.g.  C{IMailAccount}.  See the description of
+            L{Portal.login}.
+        
+        @returns: a deferred which will fire a tuple of (interface,
+            avatarAspect, logout), or the tuple itself.  The interface will be
+            one of the interfaces passed in the 'interfaces' argument.  The
+            'avatarAspect' will implement that interface.  The 'logout' object
+            is a callable which will detach the mind from the avatar.
         """
 
 
 class Portal:
+    """A mediator between clients and a realm.
+    
+    A portal is associated with one Realm and zero or more credentials checkers.
+    When a login is attempted, the portal finds the appropriate credentials
+    checker for the credentials given, invokes it, and if the credentials are
+    valid, retrieves the appropriate avatar from the Realm.
+    
+    This class is not intended to be subclassed.  Customization should be done
+    in the realm object and in the credentials checker objects.
+    """
     def __init__(self, realm, checkers=()):
         """Create a Portal to a L{IRealm}.
         """
@@ -87,7 +102,7 @@ class Portal:
         this will not be in connectionLost (such as in a web-based session), it
         will always be at the end of a user's interactive session.
         """
-        ifac = components.getInterfaces(credentials)
+        ifac = interface.providedBy(credentials)
         for i in ifac:
             c = self.checkers.get(i)
             if c is not None:
