@@ -20,7 +20,7 @@ from twisted.persisted import styles
 # Sibling Imports
 import interfaces, main
 
-class FileDescriptor(log.Logger, styles.Ephemeral):
+class FileDescriptor(log.Logger, styles.Ephemeral, object):
     """An object which can be operated on by select().
 
     This is an abstract superclass of all objects which may be notified when
@@ -37,7 +37,7 @@ class FileDescriptor(log.Logger, styles.Ephemeral):
     _writeDisconnected = False
     dataBuffer = ""
     offset = 0
-    
+
     SEND_LIMIT = 128*1024
 
     implements(interfaces.IProducer, interfaces.IReadWriteDescriptor,
@@ -49,7 +49,7 @@ class FileDescriptor(log.Logger, styles.Ephemeral):
         self.reactor = reactor
         self._tempDataBuffer = [] # will be added to dataBuffer in doWrite
         self._tempDataLen = 0
-    
+
     def connectionLost(self, reason):
         """The connection was lost.
 
@@ -68,7 +68,7 @@ class FileDescriptor(log.Logger, styles.Ephemeral):
             self.producer = None
         self.stopReading()
         self.stopWriting()
-    
+
     def writeSomeData(self, data):
         """Write as much as possible of the given data, immediately.
 
@@ -80,7 +80,7 @@ class FileDescriptor(log.Logger, styles.Ephemeral):
 
         raise NotImplementedError("%s does not implement writeSomeData" %
                                   reflect.qual(self.__class__))
-    
+
     def doRead(self):
         raise NotImplementedError("%s does not implement doRead" %
                                   reflect.qual(self.__class__))
@@ -100,7 +100,7 @@ class FileDescriptor(log.Logger, styles.Ephemeral):
             self.offset = 0
             self._tempDataBuffer = []
             self._tempDataLen = 0
-            
+
         # Send as much data as you can.
         if self.offset:
             l = self.writeSomeData(buffer(self.dataBuffer, self.offset))
@@ -147,7 +147,7 @@ class FileDescriptor(log.Logger, styles.Ephemeral):
     def _closeWriteConnection(self):
         # override in subclasses
         pass
-    
+
     def writeConnectionLost(self, reason):
         # in current code should never be called
         self.connectionLost(reason)
@@ -155,16 +155,14 @@ class FileDescriptor(log.Logger, styles.Ephemeral):
     def readConnectionLost(self, reason):
         # override in subclasses
         self.connectionLost(reason)
-    
+
     def write(self, data):
         """Reliably write some data.
 
-        If there is no buffered data this tries to write this data immediately,
-        otherwise this adds data to be written the next time this file descriptor is
-        ready for writing.
+        The data is buffered until his file descriptor is ready for writing.
         """
         if isinstance(data, unicode): # no, really, I mean it
-            raise TypeError("Data must be not be unicode")
+            raise TypeError("Data must not be unicode")
         if not self.connected or self._writeDisconnected:
             return
         if data:
@@ -187,19 +185,21 @@ class FileDescriptor(log.Logger, styles.Ephemeral):
                 self.producerPaused = 1
                 self.producer.pauseProducing()
         self.startWriting()
-    
+
     def loseConnection(self, _connDone=failure.Failure(main.CONNECTION_DONE)):
         """Close the connection at the next available opportunity.
 
-        Call this to cause this FileDescriptor to lose its connection; if this is in
-        the main loop, it will lose its connection as soon as it's done
-        flushing its write buffer; otherwise, it will wake up the main thread
-        and lose the connection immediately.
+        Call this to cause this FileDescriptor to lose its connection.  It will
+        first write any data that it has buffered.
 
-        If you have a producer registered, the connection won't be closed until the
-        producer is finished. Therefore, make sure you unregister your producer
-        when it's finished, or the connection will never close.
+        If there is data buffered yet to be written, this method will cause the
+        transport to lose its connection as soon as it's done flushing its
+        write buffer.  If you have a producer registered, the connection won't
+        be closed until the producer is finished. Therefore, make sure you
+        unregister your producer when it's finished, or the connection will
+        never close.
         """
+
         if self.connected and not self.disconnecting:
             if self._writeDisconnected:
                 # doWrite won't trigger the connection close anymore
@@ -214,7 +214,7 @@ class FileDescriptor(log.Logger, styles.Ephemeral):
     def loseWriteConnection(self):
         self._writeDisconnecting = True
         self.startWriting()
-    
+
     def stopReading(self):
         """Stop waiting for read availability.
 
@@ -308,8 +308,6 @@ class FileDescriptor(log.Logger, styles.Ephemeral):
         """
         return -1
 
-components.backwardsCompatImplements(FileDescriptor)
-
 
 def isIPAddress(addr):
     parts = string.split(addr, '.')
