@@ -1,18 +1,6 @@
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """Interface documentation.
 
@@ -28,7 +16,7 @@ from twisted.python.components import Interface
 class IAddress(Interface):
     """An address, e.g. a TCP (host, port).
 
-    Default implementations are in twisted.internet.address.
+    Default implementations are in L{twisted.internet.address}.
     """
 
 
@@ -61,11 +49,11 @@ class IConnector(Interface):
 
 
 class IResolverSimple(Interface):
-    def getHostByName(self, name, timeout = 10):
+    def getHostByName(self, name, timeout = (1, 3, 11, 45)):
         """Resolve the domain name C{name} into an IP address.
 
         @type name: C{str}
-        @type timeout: C{int}
+        @type timeout: C{tuple}
         @rtype: L{twisted.internet.defer.Deferred}
         @return: The callback of the Deferred that is returned will be
         passed a string that represents the IP address of the specified
@@ -174,7 +162,7 @@ class IReactorArbitrary(Interface):
 
 class IReactorTCP(Interface):
 
-    def listenTCP(self, port, factory, backlog=5, interface=''):
+    def listenTCP(self, port, factory, backlog=50, interface=''):
         """Connects a given protocol factory to the given numeric TCP/IP port.
 
         @param port: a port number on which to listen
@@ -185,7 +173,7 @@ class IReactorTCP(Interface):
 
         @param interface: the hostname to bind to, defaults to '' (all)
 
-        @returns: an object that satisfies the IListeningPort interface
+        @returns: an object that satisfies the L{IListeningPort} interface
 
         @raise CannotListenError: as defined here L{twisted.internet.error.CannotListenError}, if it
            cannot listen on this port (e.g., it cannot bind to the required port
@@ -209,7 +197,7 @@ class IReactorTCP(Interface):
 
         @returns:  An object implementing L{IConnector}. This connector will call
            various callbacks on the factory when a connection is made,
-           failed, or lost - see L{ClientFactory<twisted.internet.protocol.ServerFactory>} docs for details.
+           failed, or lost - see L{ClientFactory<twisted.internet.protocol.ClientFactory>} docs for details.
         """
 
 class IReactorSSL(Interface):
@@ -234,7 +222,7 @@ class IReactorSSL(Interface):
         @returns: an L{IConnector}.
         """
 
-    def listenSSL(self, port, factory, contextFactory, backlog=5, interface=''):
+    def listenSSL(self, port, factory, contextFactory, backlog=50, interface=''):
         """
         Connects a given protocol factory to the given numeric TCP/IP port.
         The connection is a SSL one, using contexts created by the context
@@ -272,7 +260,7 @@ class IReactorUNIX(Interface):
         @returns: an L{IConnector}.
         """
 
-    def listenUNIX(self, address, factory, backlog=5, mode=0666, wantPID=0):
+    def listenUNIX(self, address, factory, backlog=50, mode=0666, wantPID=0):
         """Listen on a UNIX socket.
 
         @param address: a path to a unix socket on the filesystem.
@@ -347,9 +335,10 @@ class IReactorMulticast(Interface):
     without backwards compatability. Suggestions are welcome.
     """
 
-    def listenMulticast(self, port, protocol, interface='', maxPacketSize=8192):
+    def listenMulticast(self, port, protocol, interface='', maxPacketSize=8192, listenMultiple=False):
         """Connects a given L{DatagramProtocol<twisted.internet.protocol.DatagramProtocol>} to the given numeric UDP port.
 
+        @param listenMultiple: boolean indicating whether multiple sockets can bind to same UDP port.
         @returns: object conforming to L{IListeningPort}.
         """
 
@@ -386,6 +375,11 @@ class IReactorProcess(Interface):
                        (Not available on all systems.)
 
         @see: L{twisted.internet.protocol.ProcessProtocol}
+
+        @returns: An L{IProcessTransport} object.
+
+        @raise OSError: Raised with errno EAGAIN or ENOMEM if there are
+        insufficient system resources to create a new process.
         """
 
 class IReactorTime(Interface):
@@ -476,9 +470,8 @@ class IDelayedCall(Interface):
 
     def active(self):
         """
-        @returns: A bool representing whether or not this call has been called
-                  or cancelled. (True == This DelayedCall has not been called or
-                  cancelled. False, otherwise).
+        @returns: True if this call is still active, False if it has been
+            called or cancelled.
         """
 
 class IReactorThreads(Interface):
@@ -543,6 +536,12 @@ class IReactorCore(Interface):
         This is most useful in applications where the UI is being drawn "as
         fast as possible", such as games. All pending L{IDelayedCall}s will
         be called.
+
+        The reactor must have been started (via the run() method) prior to
+        any invocations of this method.  It must also be stopped manually
+        after the last call to this method (via the stop() method).  This
+        method is not re-entrant: you must not call it recursively; in
+        particular, you must not call it while the reactor is running.
         """
 
     def fireSystemEvent(self, eventType):
@@ -618,6 +617,8 @@ class IReactorPluggableResolver(Interface):
 
         @type resolver: An object implementing the L{IResolverSimple} interface
         @param resolver: The new resolver to use.
+        
+        @return: The previously installed resolver.
         """
 
 
@@ -656,6 +657,13 @@ class IReactorFDSet(Interface):
         @returns: C{None}.
         """
 
+    def removeAll(self):
+        """Remove all readers and writers, and return the list of
+        L{IReadDescriptor}s and L{IWriteDescriptor}s removed.
+        
+        Should not remove reactor internal reactor connections (like a waker).
+        """
+
 
 class IListeningPort(Interface):
     """A listening port.
@@ -694,7 +702,24 @@ class IFileDescriptor(Interface):
         Returns: the platform-specified representation of a file-descriptor
         number.
         """
+        
+    def connectionLost(self, reason):
+        """Called when the connection was lost.
 
+        This is called when the connection on a selectable object has been
+        lost.  It will be called whether the connection was closed explicitly,
+        an exception occurred in an event handler, or the other end of the
+        connection closed it first.
+
+        See also L{IHalfCloseableDescriptor} if your descriptor wants to be
+        notified separately of the two halves of the connection being closed.
+        
+        @param reason: A failure instance indicating the reason why the
+         connection was lost.  L{twisted.internet.error.ConnectionLost} and
+         L{twisted.internet.error.ConnectionDone} are of special note, but
+         the failure may be of other classes as well.
+        """
+    
 class IReadDescriptor(IFileDescriptor):
 
     def doRead(self):
@@ -714,6 +739,16 @@ class IReadWriteDescriptor(IReadDescriptor, IWriteDescriptor):
     """
 
 
+class IHalfCloseableDescriptor(Interface):
+    """A descriptor that can be half-closed."""
+
+    def writeConnectionLost(reason):
+        """Indicates write connection was lost."""
+
+    def readConnectionLost(reason):
+        """Indicates read connection was lost."""
+
+
 class ISystemHandle(Interface):
     """An object that wraps a networking OS-specific handle."""
 
@@ -857,23 +892,52 @@ class IProtocol(Interface):
         """
 
 
+class IHalfCloseableProtocol(Interface):
+    """Implemented to indicate they want notification of half-closes.
+
+    TCP supports the notion of half-closing the connection, e.g.
+    closing the write side but still not stopping reading. A protocol
+    that implements this interface will be notified of such events,
+    instead of having connectionLost called.
+    """
+
+    def readConnectionLost():
+        """Notification of the read connection being closed.
+
+        This indicates peer did half-close of write side. It is now
+        the responsiblity of the this protocol to call
+        loseConnection().  In addition, the protocol MUST make sure a
+        reference to it still exists (i.e. by doing a callLater with
+        one of its methods, etc.)  as the reactor will only have a
+        reference to it if it is writing.
+
+        If the protocol does not do so, it might get garbage collected
+        without the connectionLost method ever being called.
+        """
+
+    def writeConnectionLost():
+        """Notification of the write connection being closed.
+
+        This will never be called for TCP connections as TCP does not
+        support notification of this type of half-close.
+        """    
+
+
 class IProtocolFactory(Interface):
     """Interface for protocol factories.
     """
 
     def buildProtocol(self, addr):
-        """Return an object implementing L{IProtocol}, or None.
-
-        This method will be called when a connection has been established
-        to addr.
+        """Called when a connection has been established to addr.
 
         If None is returned, the connection is assumed to have been refused,
         and the Port will close the connection.
 
-        TODO:
-          - Document 'addr' argument -- what format is it in?
-          - Is the phrase \"incoming server connection\" correct when Factory
-            is a ClientFactory?
+        @param addr: The address of the newly-established connection
+        @type addr: (host, port)
+        @returns: None if the connection was refused, otherwise an object
+        implementing IProtocol.
+        @rtype: None or L{IProtocol}.
         """
 
     def doStart(self):
@@ -896,7 +960,8 @@ class ITransport(Interface):
     """
 
     def write(self, data):
-        """Write some data to the physical connection, in sequence.
+        """Write some data to the physical connection, in sequence, in a
+        non-blocking fashion.
 
         If possible, make sure that it is all written.  No data will
         ever be lost, although (obviously) the connection may be closed
@@ -913,6 +978,9 @@ class ITransport(Interface):
 
     def loseConnection(self):
         """Close my connection, after writing all pending data.
+
+        Note that if there is a registered producer on a transport it
+        will not be closed until the producer has been unregistered.
         """
 
     def getPeer(self):
@@ -935,6 +1003,16 @@ class ITransport(Interface):
 class ITCPTransport(ITransport):
     """A TCP based transport."""
 
+    def loseWriteConnection(self):
+        """Half-close the write side of a TCP connection.
+        
+        If the protocol this is attached to implements
+        IHalfCloseableProtocol, it will get notified when the
+        operation is done. When closing write connection, as with
+        loseConnection this will only happen when buffer has emptied
+        and there is no registered producer.
+        """
+    
     def getTcpNoDelay(self):
         """Return if TCP_NODELAY is enabled."""
 
@@ -981,7 +1059,10 @@ class ISSLTransport(ITCPTransport):
 
 
 class IProcessTransport(ITransport):
-    """A process transport."""
+    """A process transport.
+
+    @ivar pid: The Process-ID of this process.
+    """
 
     def closeStdin(self):
         """Close stdin after all data has been written out."""
@@ -992,6 +1073,12 @@ class IProcessTransport(ITransport):
     def closeStderr(self):
         """Close stderr."""
 
+    def closeChildFD(self, descriptor):
+        """
+        Close a file descriptor which is connected to the child process, identified
+        by its FD in the child process.
+        """
+
     def loseConnection(self):
         """Close stdin, stderr and stdout."""
 
@@ -1005,6 +1092,9 @@ class IProcessTransport(ITransport):
               if possible.
           - an integer, where it represents a POSIX
               signal ID.
+
+        @raise twisted.internet.error.ProcessExitedAlready: The process has
+        already exited.
         """
 
 
@@ -1034,7 +1124,9 @@ class IUDPTransport(Interface):
         @param addr: a tuple of (ip, port). For connected transports must
                      be the address the transport is connected to, or None.
                      In non-connected mode this is mandatory.
-        @raise L{MessageLengthError<twisted.internet.error.MessageLengthError>}
+
+        @raise twisted.internet.error.MessageLengthError: C{packet} was too
+        long.
         """
 
     def connect(self, host, port):
@@ -1052,6 +1144,13 @@ class IUDPTransport(Interface):
     def getHost(self):
         """Returns IPv4Address."""
 
+    def stopListening(self):
+        """Stop listening on this port.
+
+        If it does not complete immediately, will return Deferred that fires
+        upon completion.
+        """
+
 
 class IUDPConnectedTransport(Interface):
     """DEPRECATED. Transport for UDP ConnectedPacketProtocols."""
@@ -1111,7 +1210,11 @@ class IMulticastTransport(Interface):
         """Set time to live on multicast packets."""
 
     def joinGroup(self, addr, interface=""):
-        """Join a multicast group. Returns Deferred of success."""
+        """Join a multicast group. Returns Deferred of success or failure.
+
+        If an error occurs, the returned Deferred will fail with
+        L{error.MulticastJoinError}.
+        """
 
     def leaveGroup(self, addr, interface=""):
         """Leave multicast group, return Deferred of success."""
