@@ -66,6 +66,22 @@ def unregisterReapProcessHandler(pid, process):
     del reapProcessHandlers[pid]
 
 
+def detectLinuxBrokenPipeBehavior():
+    global brokenLinuxPipeBehavior
+    r,w = os.pipe()
+    os.write(w, 'a')
+    reads, writes, exes = select.select([w], [], [], 0)
+    if reads:
+        # Linux < 2.6.11 says a write-only pipe is readable.
+        brokenLinuxPipeBehavior = True
+    else:
+        brokenLinuxPipeBehavior = False
+    os.close(r)
+    os.close(w)
+    
+# Call at import time
+detectLinuxBrokenPipeBehavior()
+
 class ProcessWriter(abstract.FileDescriptor):
     """(Internal) Helper class to write into a Process's input pipe.
 
@@ -74,15 +90,33 @@ class ProcessWriter(abstract.FileDescriptor):
     """
     connected = 1
     ic = 0
-
-    def __init__(self, reactor, proc, name, fileno):
+    enableReadHack = False
+    
+    def __init__(self, reactor, proc, name, fileno, forceReadHack=False):
         """Initialize, specifying a Process instance to connect to.
         """
         abstract.FileDescriptor.__init__(self, reactor)
+        fdesc.setNonBlocking(fileno)
         self.proc = proc
         self.name = name
         self.fd = fileno
-
+        
+        if forceReadHack:
+            self.enableReadHack = True
+        else:
+            # Detect if this fd is actually a write-only fd. If it's
+            # valid to read, don't try to detect closing via read.
+            # This really only means that we cannot detect a TTY's write
+            # pipe being closed.
+            try:
+                os.read(self.fileno(), 0)
+            except OSError:
+                # It's a write-only pipe end, enable hack
+                self.enableReadHack = True
+            
+        if self.enableReadHack:
+            self.startReading()
+            
     def fileno(self):
         """Return the fileno() of my process's stdin.
         """
@@ -94,7 +128,7 @@ class ProcessWriter(abstract.FileDescriptor):
         """
         try:
             rv = os.write(self.fd, data)
-            if rv == len(data):
+            if rv == len(data) and self.enableReadHack:
                 self.startReading()
             return rv
         except IOError, io:
@@ -129,16 +163,21 @@ class ProcessWriter(abstract.FileDescriptor):
         If the reactor is pollreactor, and the fd is > 1024, this will fail.
         (only occurs on broken versions of linux, though).
         """
-        fd = self.fd
-        r, w, x = select.select([fd], [fd], [], 0)
-        if r and w:
-            return CONNECTION_LOST
-
+        if self.enableReadHack:
+            if brokenLinuxPipeBehavior:
+                fd = self.fd
+                r, w, x = select.select([fd], [fd], [], 0)
+                if r and w:
+                    return CONNECTION_LOST
+            else:
+                return CONNECTION_LOST
+        else:
+            self.stopReading()
+        
     def connectionLost(self, reason):
         """See abstract.FileDescriptor.connectionLost.
         """
         abstract.FileDescriptor.connectionLost(self, reason)
-        os.close(self.fd)
         self.proc.childConnectionLost(self.name, reason)
 
 
@@ -154,10 +193,12 @@ class ProcessReader(abstract.FileDescriptor):
         """Initialize, specifying a process to connect to.
         """
         abstract.FileDescriptor.__init__(self, reactor)
+        fdesc.setNonBlocking(fileno)
         self.proc = proc
         self.name = name
         self.fd = fileno
-
+        self.startReading()
+        
     def fileno(self):
         """Return the fileno() of my process's stderr.
         """
@@ -176,7 +217,7 @@ class ProcessReader(abstract.FileDescriptor):
         return fdesc.readFromFD(self.fd, self.dataReceived)
 
     def dataReceived(self, data):
-        self.proto.childDataReceived(self.name, data)
+        self.proc.childDataReceived(self.name, data)
 
     def loseConnection(self):
         if self.connected and not self.disconnecting:
@@ -189,7 +230,6 @@ class ProcessReader(abstract.FileDescriptor):
         ProcessProtocol).
         """
         abstract.FileDescriptor.connectionLost(self, reason)
-        os.close(self.fd)
         self.proc.childConnectionLost(self.name, reason)
 
 
@@ -209,6 +249,9 @@ class Process(styles.Ephemeral):
     debug = False
     debug_child = False
 
+    status = -1
+    pid = None
+
     def __init__(self, reactor, command, args, environment, path, proto,
                  uid=None, gid=None, childFDs=None):
         """Spawn an operating-system process.
@@ -222,26 +265,27 @@ class Process(styles.Ephemeral):
         nuances of setXXuid on UNIX: it will assume that either your effective
         or real UID is 0.)
 
-        @param childFDs: a dictionary mapping
-            fd_in_child -> current_fd_in_parent/'r'/'w'
+        @param childFDs: a dictionary mapping fd_in_child to
+                         current_fd_in_parent/'r'/'w'.
 
-             If the value is a number, it specifies one of the parent's fds
-             that will be remapped to the child's fd. This is useful for
-             things like inetd and shell-like file redirection.
+                         If the value is a number, it specifies one of the
+                         parent's fds that will be remapped to the child's
+                         fd. This is useful for things like inetd and
+                         shell-like file redirection.
 
-             If it is the string 'r', a pipe will be created and attached to
-             the child at that fd number, and the parent will be able to
-             read from the pipe. This is useful for the child's stdout and
-             stderr.
+                         If it is the string 'r', a pipe will be created and
+                         attached to the child at that fd number, and the
+                         parent will be able to read from the pipe. This is
+                         useful for the child's stdout and stderr.
 
-             If it is the string 'w', a pipe will be created and attached,
-             and the parent will be able to write into that pipe. This is
-             useful for the child's stdin.
+                         If it is the string 'w', a pipe will be created and
+                         attached, and the parent will be able to write into
+                         that pipe. This is useful for the child's stdin.
 
-            If childFDs is not passed, the default behaviour is to use a
-            mapping that opens the usual stdin/stdout/stderr pipes.
+                         If childFDs is not passed, the default behaviour is to
+                         use a mapping that opens the usual stdin/stdout/stderr
+                         pipes.
         """
-
         if not proto:
             assert 'r' not in childFDs.values()
             assert 'w' not in childFDs.values()
@@ -299,14 +343,12 @@ class Process(styles.Ephemeral):
                 if debug: print "readFD=%d, writeFD%d" % (readFD, writeFD)
                 fdmap[childFD] = writeFD     # child writes to this
                 helpers[childFD] = readFD    # parent reads from this
-                fdesc.setNonBlocking(readFD)
             elif target == "w":
                 # we need a pipe that the parent can write to
                 readFD, writeFD = os.pipe()
                 if debug: print "readFD=%d, writeFD=%d" % (readFD, writeFD)
                 fdmap[childFD] = readFD      # child reads from this
                 helpers[childFD] = writeFD   # parent writes to this
-                fdesc.setNonBlocking(writeFD)
             else:
                 assert type(target) == int, '%r should be an int' % (target,)
                 fdmap[childFD] = target      # parent ignores this
@@ -361,26 +403,20 @@ class Process(styles.Ephemeral):
             os.setreuid(curruid, cureuid)
         self.status = -1 # this records the exit status of the child
 
+        self.proto = proto
+        
         # arrange for the parent-side pipes to be read and written
         for childFD, parentFD in helpers.items():
             os.close(fdmap[childFD])
 
             if childFDs[childFD] == "r":
                 reader = ProcessReader(reactor, self, childFD, parentFD)
-                reader.proto = proto
                 self.pipes[childFD] = reader
-                reader.startReading()
 
             if childFDs[childFD] == "w":
-                writer = ProcessWriter(reactor, self, childFD, parentFD)
-                writer.proto = proto
+                writer = ProcessWriter(reactor, self, childFD, parentFD, forceReadHack=True)
                 self.pipes[childFD] = writer
-                # we do startReading here to watch for EOF. We won't do an
-                # actual .startWriting until some data has been written to
-                # the transmit buffer.
-                writer.startReading()
 
-        self.proto = proto
         try:
             # the 'transport' is used for some compatibility methods
             if self.proto is not None:
@@ -398,22 +434,25 @@ class Process(styles.Ephemeral):
         'command 2>&1' would correspond to an fdmap of {0:0, 1:1, 2:1}.
         'command >foo.txt' would be {0:0, 1:os.open('foo.txt'), 2:2}.
 
-        Step 1: close all file descriptors that aren't values of fdmap.
-        This means 0 .. maxfds.
+        This is accomplished in two steps::
 
-        Step 2: for each childFD:
-         if fdmap[childFD] == childFD, the descriptor is already in place.
-         Make sure the CLOEXEC flag is not set, then delete the entry from
-         fdmap.
+            1. close all file descriptors that aren't values of fdmap.  This means
+               0 .. maxfds.
 
-         if childFD is in fdmap.values(), then the target descriptor is
-         busy. Use os.dup() to move it elsewhere, update all fdmap[childFD]
-         items that point to it, then close the original. Then fall through
-         to the next case.
+            2. for each childFD::
 
-         now fdmap[childFD] is not in fdmap.values(), and is free. Use
-         os.dup2() to move it to the right place, then close the original.
+                 - if fdmap[childFD] == childFD, the descriptor is already in
+                   place.  Make sure the CLOEXEC flag is not set, then delete the
+                   entry from fdmap.
 
+                 - if childFD is in fdmap.values(), then the target descriptor is
+                   busy. Use os.dup() to move it elsewhere, update all
+                   fdmap[childFD] items that point to it, then close the
+                   original. Then fall through to the next case.
+
+                 - now fdmap[childFD] is not in fdmap.values(), and is free. Use
+                   os.dup2() to move it to the right place, then close the
+                   original.
         """
 
         debug = self.debug_child
@@ -555,6 +594,7 @@ class Process(styles.Ephemeral):
         self.closeStdin()
         self.closeStderr()
         self.closeStdout()
+
     def write(self,data):
         """Call this to write to standard input on this process.
         
@@ -563,6 +603,33 @@ class Process(styles.Ephemeral):
         if self.pipes.has_key(0):
             self.pipes[0].write(data)
 
+    def registerProducer(self, producer, streaming):
+        """Call this to register producer for standard input.
+
+        If there is no standard input producer.stopProducing() will
+        be called immediately.
+        """
+        if self.pipes.has_key(0):
+            self.pipes[0].registerProducer(producer, streaming)
+        else:
+            producer.stopProducing()
+
+    def unregisterProducer(self):
+        """Call this to unregister producer for standard input."""
+        if self.pipes.has_key(0):
+            self.pipes[0].unregisterProducer()
+    
+    def writeSequence(self, seq):
+        """Call this to write to standard input on this process.
+
+        NOTE: This will silently lose data if there is no standard input.
+        """
+        if self.pipes.has_key(0):
+            self.pipes[0].writeSequence(seq)
+
+    def childDataReceived(self, name, data):
+        self.proto.childDataReceived(name, data)
+        
     def signalProcess(self, signalID):
         if signalID in ('HUP', 'STOP', 'INT', 'KILL', 'TERM'):
             signalID = getattr(signal, 'SIG'+signalID)
@@ -583,6 +650,7 @@ class Process(styles.Ephemeral):
     def childConnectionLost(self, childFD, reason):
         # this is called when one of the helpers (ProcessReader or
         # ProcessWriter) notices their pipe has been closed
+        os.close(self.pipes[childFD].fileno())
         del self.pipes[childFD]
         try:
             self.proto.childConnectionLost(childFD)
@@ -596,7 +664,6 @@ class Process(styles.Ephemeral):
         #  all writers have indicated an error status, AND
         #  all readers have indicated EOF
         # This insures that we've gathered all output from the process.
-
         if self.pipes:
             #print "maybe, but pipes still", self.pipes.keys()
             return
@@ -623,10 +690,15 @@ class Process(styles.Ephemeral):
         except:
             log.err()
 
+    def __repr__(self):
+        return "<%s pid=%s status=%s>" % (self.__class__.__name__,
+                                          self.pid, self.status)
 
 class PTYProcess(abstract.FileDescriptor, styles.Ephemeral):
     """An operating-system Process that uses PTY support."""
-
+    status = -1
+    pid = None
+    
     def __init__(self, reactor, command, args, environment, path, proto,
                  uid=None, gid=None, usePTY=None):
         """Spawn an operating-system process.
@@ -741,7 +813,7 @@ class PTYProcess(abstract.FileDescriptor, styles.Ephemeral):
         try:
             pid, status = os.waitpid(self.pid, os.WNOHANG)
         except OSError, e:
-            if e.errno == 10: # no child process
+            if e.errno == errno.ECHILD: # no child process
                 pid = None
             else:
                 raise
@@ -829,3 +901,7 @@ class PTYProcess(abstract.FileDescriptor, styles.Ephemeral):
             if io.args[0] == errno.EAGAIN:
                 return 0
             return CONNECTION_LOST
+
+    def __repr__(self):
+        return "<%s pid=%s status=%s>" % (self.__class__.__name__,
+                                          self.pid, self.status)
