@@ -1,20 +1,8 @@
 # -*- test-case-name: twisted.test.test_factories -*-
 #
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """Standard implementations of Twisted protocol-related interfaces.
 
@@ -27,9 +15,10 @@ Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
 """
 
 import random
+from zope.interface import implements
 
 # Twisted Imports
-from twisted.python import log, failure
+from twisted.python import log, failure, components
 from twisted.internet import interfaces, error, defer
 
 
@@ -40,29 +29,38 @@ class Factory:
     self.protocol.
     """
 
-    __implements__ = (interfaces.IProtocolFactory,)
+    implements(interfaces.IProtocolFactory)
 
     # put a subclass of Protocol here:
     protocol = None
 
     numPorts = 0
-    noisy = "sure, why not"
+    noisy = True
 
     def doStart(self):
-        """Make sure startFactory is called."""
+        """Make sure startFactory is called.
+
+        Users should not call this function themselves!
+        """
         if not self.numPorts:
             if self.noisy:
-                log.msg("Starting factory %s" % self)
+                log.msg("Starting factory %r" % self)
             self.startFactory()
         self.numPorts = self.numPorts + 1
 
     def doStop(self):
-        """Make sure stopFactory is called."""
-        assert self.numPorts > 0
+        """Make sure stopFactory is called.
+
+        Users should not call this function themselves!
+        """
+        if self.numPorts == 0:
+            # this shouldn't happen, but does sometimes and this is better
+            # than blowing up in assert as we did previously.
+            return
         self.numPorts = self.numPorts - 1
         if not self.numPorts:
             if self.noisy:
-                log.msg("Stopping factory %s" % self)
+                log.msg("Stopping factory %r" % self)
             self.stopFactory()
 
     def startFactory(self):
@@ -79,11 +77,12 @@ class Factory:
     def stopFactory(self):
         """This will be called before I stop listening on all Ports/Connectors.
 
-        This can be used to perform 'shutdown' tasks such as disconnecting
+        This can be overridden to perform 'shutdown' tasks such as disconnecting
         database connections, closing files, etc.
 
         It will be called, for example, before an application shuts down,
-        if it was connected to a port.
+        if it was connected to a port. User code should not call this function
+        directly.
         """
 
     def buildProtocol(self, addr):
@@ -137,26 +136,36 @@ class ClientFactory(Factory):
 class _InstanceFactory(ClientFactory):
     """Factory used by ClientCreator."""
 
+    noisy = False
+    
     def __init__(self, reactor, instance, deferred):
         self.reactor = reactor
         self.instance = instance
         self.deferred = deferred
 
+    def __repr__(self):
+        return "<ClientCreator factory: %r>" % (self.instance, )
+    
     def buildProtocol(self, addr):
         self.reactor.callLater(0, self.deferred.callback, self.instance)
+        del self.deferred
         return self.instance
 
     def clientConnectionFailed(self, connector, reason):
         self.reactor.callLater(0, self.deferred.errback, reason)
+        del self.deferred
 
 
 class ClientCreator:
-    """Useful for cases when we don't really need a factory.
+    """Client connections that do not require a factory.
 
-    Mainly this is when there is no shared state between protocol instances
-    (e.g. protocols that work in a way similar to httplib or smtplib).
-
-    The API for this class is not stable.
+    The various connect* methods create a protocol instance using the given
+    protocol class and arguments, and connect it, returning a Deferred of the
+    resulting protocol instance.
+    
+    Useful for cases when we don't really need a factory.  Mainly this
+    is when there is no shared state between protocol instances, and no need
+    to reconnect.
     """
 
     def __init__(self, reactor, protocolClass, *args, **kwargs):
@@ -172,11 +181,20 @@ class ClientCreator:
         self.reactor.connectTCP(host, port, f, timeout=timeout, bindAddress=bindAddress)
         return d
 
-    def connectUNIX(self, address, timeout = 30):
+    def connectUNIX(self, address, timeout = 30, checkPID=0):
+        """Connect to Unix socket, return Deferred of resulting protocol instance."""
         d = defer.Deferred()
         f = _InstanceFactory(self.reactor, self.protocolClass(*self.args, **self.kwargs), d)
-        self.reactor.connectUNIX(address, f, timeout = timeout)
+        self.reactor.connectUNIX(address, f, timeout = timeout, checkPID=checkPID)
         return d
+    
+    def connectSSL(self, host, port, contextFactory, timeout=30, bindAddress=None):
+        """Connect to SSL server, return Deferred of resulting protocol instance."""
+        d = defer.Deferred()
+        f = _InstanceFactory(self.reactor, self.protocolClass(*self.args, **self.kwargs), d)
+        self.reactor.connectSSL(host, port, f, contextFactory, timeout=timeout, bindAddress=bindAddress)
+        return d
+
 
 class ReconnectingClientFactory(ClientFactory):
     """My clients auto-reconnect with an exponential back-off.
@@ -193,7 +211,7 @@ class ReconnectingClientFactory(ClientFactory):
     maxDelay = 3600
     initialDelay = 1.0
     # Note: These highly sensitive factors have been precisely measured by
-    # the National Institute of Science and Technology.  Take extereme care
+    # the National Institute of Science and Technology.  Take extreme care
     # in altering them, or you may damage your Internet!
     factor = 2.7182818284590451 # (math.e)
     # Phi = 1.6180339887498948 # (Phi is acceptable for use as a
@@ -211,8 +229,7 @@ class ReconnectingClientFactory(ClientFactory):
     def clientConnectionFailed(self, connector, reason):
         if self.continueTrying:
             self.connector = connector
-            if not reason.check(error.UserError):
-                self.retry()
+            self.retry()
 
     def clientConnectionLost(self, connector, unused_reason):
         if self.continueTrying:
@@ -223,7 +240,8 @@ class ReconnectingClientFactory(ClientFactory):
         """Have this connector connect again, after a suitable delay.
         """
         if not self.continueTrying:
-            log.msg("Abandoning %s on explicit request" % (connector,))
+            if self.noisy:
+                log.msg("Abandoning %s on explicit request" % (connector,))
             return
 
         if connector is None:
@@ -234,8 +252,9 @@ class ReconnectingClientFactory(ClientFactory):
 
         self.retries += 1
         if self.maxRetries is not None and (self.retries > self.maxRetries):
-            log.msg("Abandoning %s after %d retries." %
-                    (connector, self.retries))
+            if self.noisy:
+                log.msg("Abandoning %s after %d retries." %
+                        (connector, self.retries))
             return
 
         self.delay = min(self.delay * self.factor, self.maxDelay)
@@ -243,7 +262,8 @@ class ReconnectingClientFactory(ClientFactory):
             self.delay = random.normalvariate(self.delay,
                                               self.delay * self.jitter)
 
-        log.msg("%s will retry in %d seconds" % (connector, self.delay,))
+        if self.noisy:
+            log.msg("%s will retry in %d seconds" % (connector, self.delay,))
         from twisted.internet import reactor
 
         def reconnector():
@@ -326,9 +346,10 @@ class BaseProtocol:
 connectionDone=failure.Failure(error.ConnectionDone())
 connectionDone.cleanFailure()
 
+
 class Protocol(BaseProtocol):
 
-    __implements__ = (interfaces.IProtocol,)
+    implements(interfaces.IProtocol)
 
     def dataReceived(self, data):
         """Called whenever data is received.
@@ -353,23 +374,12 @@ class Protocol(BaseProtocol):
         @type reason: L{twisted.python.failure.Failure}
         """
 
-    def connectionFailed(self):
-        """(Deprecated)
-
-        This used to be called when the connection was not properly
-        established.
-        """
-        import warnings
-        warnings.warn("connectionFailed is deprecated.  See new Client API",
-                      category=DeprecationWarning, stacklevel=2)
-
-from twisted.python import components
 
 class ProtocolToConsumerAdapter(components.Adapter):
     """
     This class is unstable.
     """
-    __implements__ = interfaces.IConsumer
+    implements(interfaces.IConsumer)
 
     def write(self, data):
         self.original.dataReceived(data)
@@ -387,7 +397,7 @@ class ConsumerToProtocolAdapter(components.Adapter):
     """
     This class is unstable.
     """
-    __implements__ = interfaces.IProtocol
+    implements(interfaces.IProtocol)
 
     def dataReceived(self, data):
         self.original.write(data)
@@ -540,9 +550,12 @@ class ConnectedDatagramProtocol(DatagramProtocol):
 
 class FileWrapper:
     """A wrapper around a file-like object to make it behave as a Transport.
+
+    This doesn't actually stream the file to the attached protocol,
+    and is thus useful mainly as a utility for debugging protocols.
     """
 
-    __implements__ = interfaces.ITransport
+    implements(interfaces.ITransport)
 
     closed = 0
     disconnecting = 0
@@ -591,15 +604,29 @@ class FileWrapper:
             self.handleException()
 
     def getPeer(self):
+        # XXX: According to ITransport, this should return an IAddress!
         return 'file', 'file'
 
     def getHost(self):
+        # XXX: According to ITransport, this should return an IAddress!
         return 'file'
 
     def handleException(self):
         pass
 
+    def resumeProducing(self):
+        # Never sends data anyways
+        pass
 
-__all__ = ["Factory", "ClientFactory", "ReconnectingClientFactory",
+    def pauseProducing(self):
+        # Never sends data anyways
+        pass
+    
+    def stopProducing(self):
+        self.loseConnection()
+        
+
+__all__ = ["Factory", "ClientFactory", "ReconnectingClientFactory", "connectionDone", 
            "Protocol", "ProcessProtocol", "FileWrapper", "ServerFactory",
-           "DatagramProtocol", "ConnectedDatagramProtocol", "ClientCreator"]
+           "AbstractDatagramProtocol", "DatagramProtocol", "ConnectedDatagramProtocol",
+           "ClientCreator"]
