@@ -1,18 +1,6 @@
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """
 This module provides support for Twisted to interact with the PyQt mainloop.
@@ -37,10 +25,8 @@ from qt import QSocketNotifier, QObject, SIGNAL, QTimer, QApplication
 import sys
 
 # Twisted Imports
-from twisted.python import log, threadable, failure
-
-# Sibling Imports
-import main, default
+from twisted.python import log, failure
+from twisted.internet import posixbase
 
 reads = {}
 writes = {}
@@ -60,7 +46,7 @@ class TwistedSocketNotifier(QSocketNotifier):
             self.fn = self.read
         elif type == QSocketNotifier.Write:
             self.fn = self.write
-        QObject.connect( self, SIGNAL("activated(int)"), self.fn )
+        QObject.connect(self, SIGNAL("activated(int)"), self.fn)
 
     def shutdown(self):
         QObject.disconnect(self, SIGNAL("activated(int)"), self.fn)
@@ -77,12 +63,7 @@ class TwistedSocketNotifier(QSocketNotifier):
             log.msg('Error in %s.doRead()' % w)
             log.deferr()
         if why:
-            self.reactor.removeReader(w)
-            self.reactor.removeWriter(w)
-            try:
-                w.connectionLost(failure.Failure(why))
-            except:
-                log.deferr()
+            self.reactor._disconnectSelectable(w, why, True)
         self.reactor.simulate()
 
     def write(self, sock):
@@ -107,19 +88,22 @@ class TwistedSocketNotifier(QSocketNotifier):
         self.reactor.simulate()
 
 
-# global timer
-_timer = None
-
-
-class QTReactor(default.PosixReactorBase):
+class QTReactor(posixbase.PosixReactorBase):
     """Qt based reactor."""
 
+    # Reference to a DelayedCall for self.crash() when the reactor is
+    # entered through .iterate()
+    _crashCall = None
+
+    _timer = None
+
     def __init__(self, app=None):
-        self.running = 1
-        default.PosixReactorBase.__init__(self)
+        self.running = 0
+        posixbase.PosixReactorBase.__init__(self)
         if app is None:
             app = QApplication([])
         self.qApp = app
+        self.addSystemEventTrigger('after', 'shutdown', self.cleanup)
 
     def addReader(self, reader):
         if not hasReader(reader):
@@ -140,43 +124,42 @@ class QTReactor(default.PosixReactorBase):
             del writes[writer]
 
     def removeAll(self):
-        v = reads.keys()
-        for reader in v:
-            self.removeReader(reader)
-        return v
+        return self._removeAll(reads, writes)
 
     def simulate(self):
-        global _timer
-        if _timer: _timer.stop()
+        if self._timer is not None:
+            self._timer.stop()
+            self._timer = None
+
         if not self.running:
             self.running = 1
             self.qApp.exit_loop()
             return
         self.runUntilCurrent()
 
+        if self._crashCall is not None:
+            self._crashCall.reset(0)
+
         # gah
         timeout = self.timeout()
-        if timeout is None: timeout = 1.0
+        if timeout is None:
+            timeout = 1.0
         timeout = min(timeout, 0.1) * 1010
 
-        if not _timer:
-            _timer = QTimer()
-            QObject.connect( _timer, SIGNAL("timeout()"), self.simulate )
-        _timer.start(timeout, 1)
+        if self._timer is None:
+            self._timer = QTimer()
+            QObject.connect(self._timer, SIGNAL("timeout()"), self.simulate)
+        self._timer.start(timeout, 1)
 
     def cleanup(self):
-        global _timer
-        if _timer:
-            _timer.stop()
-            _timer = None
+        if self._timer is not None:
+            self._timer.stop()
+            self._timer = None
 
-    def doIteration(self, delay=0.0):
+    def iterate(self, delay=0.0):
         log.msg(channel='system', event='iteration', reactor=self)
-        if delay is None:
-            delay = 1000
-        else:
-            delay = int(delay * 1000)
-        self.qApp.processEvents(delay)
+        self._crashCall = self.callLater(delay, self.crash)
+        self.run()
 
     def run(self, installSignalHandlers=1):
         self.running = 1
@@ -185,15 +168,17 @@ class QTReactor(default.PosixReactorBase):
         self.qApp.enter_loop()
 
     def crash(self):
+        if self._crashCall is not None:
+            if self._crashCall.active():
+                self._crashCall.cancel()
+            self._crashCall = None
         self.running = 0
 
 
 def install(app=None):
     """Configure the twisted mainloop to be run inside the qt mainloop.
     """
+    from twisted.internet import main
+
     reactor = QTReactor(app=app)
-    reactor.addSystemEventTrigger('after', 'shutdown', reactor.cleanup )
-    reactor.simulate()
-    from twisted.internet.main import installReactor
-    installReactor(reactor)
-    return reactor
+    main.installReactor(reactor)
