@@ -1,18 +1,7 @@
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001-2003 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# -*- test-case-name: twisted.test.test_sob -*-
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 #
 """Save and load Small OBjects to and from files, using various formats.
 
@@ -32,6 +21,7 @@ except ImportError:
     import StringIO
 from twisted.python import components, log, runtime
 from twisted.persisted import styles
+from zope.interface import implements
 
 # Note:
 # These encrypt/decrypt functions only work for data formats
@@ -70,7 +60,7 @@ class IPersistable(components.Interface):
 
 class Persistent:
 
-    __implements__ = IPersistable,
+    implements(IPersistable)
 
     style = "pickle"
 
@@ -116,7 +106,7 @@ class Persistent:
             ext = "tas"
         else:
             def dumpFunc(obj, file):
-                pickle.dump(obj, file, 1)
+                pickle.dump(obj, file, 2)
             ext = "tap"
         return ext, dumpFunc
 
@@ -206,11 +196,14 @@ def loadValueFromFile(filename, variable, passphrase=None):
         mode = 'rb'
     else:
         mode = 'r'
-    data = open(filename, mode).read()
-    if passphrase:
-        data = _decrypt(passphrase, data)
+    fileObj = open(filename, mode)
     d = {'__file__': filename}
-    exec data in d, d
+    if passphrase:
+        data = fileObj.read()
+        data = _decrypt(passphrase, data)
+        exec data in d, d
+    else:
+        exec fileObj in d, d
     value = d[variable]
     return value
 
