@@ -1,20 +1,8 @@
 # -*- test-case-name: twisted.test.test_protocols -*-
 
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """Basic protocols, such as line-oriented, netstring, and 32-bit-int prefixed strings.
 
@@ -29,8 +17,8 @@ import struct
 
 # Twisted imports
 from twisted.internet import protocol, defer, interfaces, error
-from twisted.python import log
-
+from twisted.python import log, components
+from zope.interface import implements
 
 LENGTH, DATA, COMMA = range(3)
 NUMBER = re.compile('(\d*)(:?)')
@@ -130,21 +118,25 @@ class SafeNetstringReceiver(NetstringReceiver):
 
 class LineOnlyReceiver(protocol.Protocol):
     """A protocol that receives only lines.
-    
-    This is purely a speed optimisation over LineReceiver, for the cases that raw mode is known to be unnecessary.
+
+    This is purely a speed optimisation over LineReceiver, for the
+    cases that raw mode is known to be unnecessary.
+
+    @cvar delimiter: The line-ending delimiter to use. By default this is
+                     '\\r\\n'.
+    @cvar MAX_LENGTH: The maximum length of a line to allow (If a
+                      sent line is longer than this, the connection is dropped).
+                      Default is 16384.
     """
     _buffer = ''
     delimiter = '\r\n'
     MAX_LENGTH = 16384
-    
+
     def dataReceived(self, data):
-        """Protocol.dataReceived.
-        Translates bytes into lines, and calls lineReceived.
-        """
-        
+        """Translates bytes into lines, and calls lineReceived."""
         lines  = (self._buffer+data).split(self.delimiter)
-        self._buffer = lines[-1]
-        for line in lines[:-1]:
+        self._buffer = lines.pop(-1)
+        for line in lines:
             if self.transport.disconnecting:
                 # this is necessary because the transport may be told to lose
                 # the connection by a line within a larger packet, and it is
@@ -152,11 +144,11 @@ class LineOnlyReceiver(protocol.Protocol):
                 # the one that told it to close.
                 return
             if len(line) > self.MAX_LENGTH:
-                return self.lineLengthExceeded(line)                
+                return self.lineLengthExceeded(line)
             else:
                 self.lineReceived(line)
         if len(self._buffer) > self.MAX_LENGTH:
-            return self.lineLengthExceeded(self._buffer)                
+            return self.lineLengthExceeded(self._buffer)
 
     def lineReceived(self, line):
         """Override this for when each line is received.
@@ -173,24 +165,40 @@ class LineOnlyReceiver(protocol.Protocol):
         Override if it needs to be dealt with in some special way.
         """
         return error.ConnectionLost('Line length exceeded')
-    
 
 
-class LineReceiver(protocol.Protocol):
+class _PauseableMixin:
+    paused = False
+
+    def pauseProducing(self):
+        self.paused = True
+        self.transport.pauseProducing()
+
+    def resumeProducing(self):
+        self.paused = False
+        self.transport.resumeProducing()
+        self.dataReceived('')
+
+    def stopProducing(self):
+        self.paused = True
+        self.transport.stopProducing()
+
+
+class LineReceiver(protocol.Protocol, _PauseableMixin):
     """A protocol that receives lines and/or raw data, depending on mode.
-    
+
     In line mode, each line that's received becomes a callback to
     L{lineReceived}.  In raw data mode, each chunk of raw data becomes a
     callback to L{rawDataReceived}.  The L{setLineMode} and L{setRawMode}
     methods switch between the two modes.
-    
+
     This is useful for line-oriented protocols such as IRC, HTTP, POP, etc.
-    
+
     @cvar delimiter: The line-ending delimiter to use. By default this is
                      '\\r\\n'.
     @cvar MAX_LENGTH: The maximum length of a line to allow (If a
                       sent line is longer than this, the connection is dropped).
-                      Default is 16834.
+                      Default is 16384.
     """
     line_mode = 1
     __buffer = ''
@@ -200,14 +208,14 @@ class LineReceiver(protocol.Protocol):
     def clearLineBuffer(self):
         """Clear buffered data."""
         self.__buffer = ""
-    
+
     def dataReceived(self, data):
         """Protocol.dataReceived.
         Translates bytes into lines, and calls lineReceived (or
         rawDataReceived, depending on mode.)
         """
         self.__buffer = self.__buffer+data
-        while self.line_mode:
+        while self.line_mode and not self.paused:
             try:
                 line, self.__buffer = self.__buffer.split(self.delimiter, 1)
             except ValueError:
@@ -218,15 +226,18 @@ class LineReceiver(protocol.Protocol):
             else:
                 linelength = len(line)
                 if linelength > self.MAX_LENGTH:
-                    line, self.__buffer = self.__buffer, ''
-                    return self.lineLengthExceeded(line)
+                    exceeded = line + self.__buffer
+                    self.__buffer = ''
+                    return self.lineLengthExceeded(exceeded)
                 why = self.lineReceived(line)
                 if why or self.transport and self.transport.disconnecting:
                     return why
         else:
-            data, self.__buffer = self.__buffer, ''
-            if data:
-                return self.rawDataReceived(data)
+            if not self.paused:
+                data=self.__buffer
+                self.__buffer=''
+                if data:
+                    return self.rawDataReceived(data)
 
     def setLineMode(self, extra=''):
         """Sets the line-mode of this receiver.
@@ -235,9 +246,13 @@ class LineReceiver(protocol.Protocol):
         you can pass in extra unhandled data, and that data will
         be parsed for lines.  Further data received will be sent
         to lineReceived rather than rawDataReceived.
+
+        Do not pass extra data if calling this function from
+        within a lineReceived callback.
         """
         self.line_mode = 1
-        return self.dataReceived(extra)
+        if extra:
+            return self.dataReceived(extra)
 
     def setRawMode(self):
         """Sets the raw mode of this receiver.
@@ -264,11 +279,16 @@ class LineReceiver(protocol.Protocol):
     def lineLengthExceeded(self, line):
         """Called when the maximum line length has been reached.
         Override if it needs to be dealt with in some special way.
+
+        The argument 'line' contains the remainder of the buffer, starting
+        with (at least some part) of the line which is too long. This may
+        be more than one line, or may be only the initial portion of the
+        line.
         """
         return self.transport.loseConnection()
 
 
-class Int32StringReceiver(protocol.Protocol):
+class Int32StringReceiver(protocol.Protocol, _PauseableMixin):
     """A receiver for int32-prefixed strings.
 
     An int32 string is a string prefixed by 4 bytes, the 32-bit length of
@@ -289,7 +309,7 @@ class Int32StringReceiver(protocol.Protocol):
         """Convert int32 prefixed strings into calls to stringReceived.
         """
         self.recvd = self.recvd + recd
-        while len(self.recvd) > 3:
+        while len(self.recvd) > 3 and not self.paused:
             length ,= struct.unpack("!i",self.recvd[:4])
             if length > self.MAX_LENGTH:
                 self.transport.loseConnection()
@@ -306,7 +326,7 @@ class Int32StringReceiver(protocol.Protocol):
         self.transport.write(struct.pack("!i",len(data))+data)
 
 
-class Int16StringReceiver(protocol.Protocol):
+class Int16StringReceiver(protocol.Protocol, _PauseableMixin):
     """A receiver for int16-prefixed strings.
 
     An int16 string is a string prefixed by 2 bytes, the 16-bit length of
@@ -326,7 +346,7 @@ class Int16StringReceiver(protocol.Protocol):
         """Convert int16 prefixed strings into calls to stringReceived.
         """
         self.recvd = self.recvd + recd
-        while len(self.recvd) > 1:
+        while len(self.recvd) > 1 and not self.paused:
             length = (ord(self.recvd[0]) * 256) + ord(self.recvd[1])
             if len(self.recvd) < length+2:
                 break
@@ -371,33 +391,33 @@ class StatefulStringProtocol:
 
 class FileSender:
     """A producer that sends the contents of a file to a consumer.
-    
+
     This is a helper for protocols that, at some point, will take a
     file-like object, read its contents, and write them out to the network,
     optionally performing some transformation on the bytes in between.
-    
+
     This API is unstable.
     """
-    __implements__ = (interfaces.IProducer,)
-    
+    implements(interfaces.IProducer)
+
     CHUNK_SIZE = 2 ** 14
-    
+
     lastSent = ''
     deferred = None
 
     def beginFileTransfer(self, file, consumer, transform = None):
         """Begin transferring a file
-        
+
         @type file: Any file-like object
         @param file: The file object to read data from
-        
+
         @type consumer: Any implementor of IConsumer
         @param consumer: The object to write data to
-        
+
         @param transform: A callable taking one string argument and returning
         the same.  All bytes read from the file are passed through this before
         being written to the consumer.
-        
+
         @rtype: C{Deferred}
         @return: A deferred whose callback will be invoked when the file has been
         completely written to the consumer.  The last byte written to the consumer
@@ -406,11 +426,11 @@ class FileSender:
         self.file = file
         self.consumer = consumer
         self.transform = transform
-        
+
+        self.deferred = deferred = defer.Deferred()
         self.consumer.registerProducer(self, False)
-        self.deferred = defer.Deferred()
-        return self.deferred
-    
+        return deferred
+
     def resumeProducing(self):
         chunk = ''
         if self.file:
@@ -422,7 +442,7 @@ class FileSender:
                 self.deferred.callback(self.lastSent)
                 self.deferred = None
             return
-        
+
         if self.transform:
             chunk = self.transform(chunk)
         self.consumer.write(chunk)
@@ -430,7 +450,7 @@ class FileSender:
 
     def pauseProducing(self):
         pass
-    
+
     def stopProducing(self):
         if self.deferred:
             self.deferred.errback(Exception("Consumer asked us to stop producing"))
