@@ -21,8 +21,6 @@ from zope.interface import implements, providedBy, directlyProvides
 
 class ProtocolWrapper(Protocol):
     """Wraps protocol instances and acts as their transport as well."""
-    
-    implements(ITransport)
 
     disconnecting = 0
 
@@ -31,12 +29,11 @@ class ProtocolWrapper(Protocol):
         self.factory = factory
 
     def makeConnection(self, transport):
-        for iface in providedBy(transport):
-            directlyProvides(self, iface)
+        directlyProvides(self, *providedBy(self) + providedBy(transport))
         Protocol.makeConnection(self, transport)
-    
+
     # Transport relaying
-    
+
     def write(self, data):
         self.transport.write(data)
 
@@ -52,7 +49,7 @@ class ProtocolWrapper(Protocol):
 
     def getHost(self):
         return self.transport.getHost()
-    
+
     def registerProducer(self, producer, streaming):
         self.transport.registerProducer(producer, streaming)
 
@@ -64,9 +61,9 @@ class ProtocolWrapper(Protocol):
 
     def __getattr__(self, name):
         return getattr(self.transport, name)
-    
+
     # Protocol relaying
-    
+
     def connectionMade(self):
         self.factory.registerProtocol(self)
         self.wrappedProtocol.makeConnection(self)
@@ -78,14 +75,12 @@ class ProtocolWrapper(Protocol):
         self.factory.unregisterProtocol(self)
         self.wrappedProtocol.connectionLost(reason)
 
-components.backwardsCompatImplements(ProtocolWrapper)
-
 
 class WrappingFactory(ClientFactory):
     """Wraps a factory and its protocols, and keeps track of them."""
-    
+
     protocol = ProtocolWrapper
-    
+
     def __init__(self, wrappedFactory):
         self.wrappedFactory = wrappedFactory
         self.protocols = {}
@@ -97,7 +92,7 @@ class WrappingFactory(ClientFactory):
     def doStop(self):
         self.wrappedFactory.doStop()
         ClientFactory.doStop(self)
-    
+
     def startedConnecting(self, connector):
         self.wrappedFactory.startedConnecting(connector)
 
@@ -108,7 +103,7 @@ class WrappingFactory(ClientFactory):
         self.wrappedFactory.clientConnectionLost(connector, reason)
 
     def buildProtocol(self, addr):
-        return self.protocol(self, self.wrappedFactory.buildProtocol(addr))    
+        return self.protocol(self, self.wrappedFactory.buildProtocol(addr))
 
     def registerProtocol(self, p):
         """Called by protocol to register itself."""
@@ -123,7 +118,7 @@ class ThrottlingProtocol(ProtocolWrapper):
     """Protocol for ThrottlingFactory."""
 
     # wrap API for tracking bandwidth
-    
+
     def write(self, data):
         self.factory.registerWritten(len(data))
         ProtocolWrapper.write(self, data)
@@ -144,12 +139,12 @@ class ThrottlingProtocol(ProtocolWrapper):
         del self.producer
         ProtocolWrapper.unregisterProducer(self)
 
-    
+
     def throttleReads(self):
-        self.transport.stopReading()
+        self.transport.pauseProducing()
 
     def unthrottleReads(self):
-        self.transport.startReading()
+        self.transport.resumeProducing()
 
     def throttleWrites(self):
         if hasattr(self, "producer"):
@@ -206,7 +201,7 @@ class ThrottlingFactory(WrappingFactory):
             throttleTime = (float(self.writtenThisSecond) / self.writeLimit) - 1.0
             self.unthrottleWritesID = reactor.callLater(throttleTime,
                                                         self.unthrottleWrites)
-        # reset for next round    
+        # reset for next round
         self.writtenThisSecond = 0
         self.checkWriteBandwidthID = reactor.callLater(1, self.checkWriteBandwidth)
 
@@ -283,7 +278,7 @@ class LimitConnectionsByPeer(WrappingFactory):
 
     def startFactory(self):
         self.peerConnections = {}
-        
+
     def buildProtocol(self, addr):
         peerHost = addr[0]
         connectionCount = self.peerConnections.get(peerHost, 0)
@@ -299,9 +294,51 @@ class LimitConnectionsByPeer(WrappingFactory):
             del self.peerConnections[peerHost]
 
 
+class LimitTotalConnectionsFactory(ServerFactory):
+    """Factory that limits the number of simultaneous connections.
+
+    API Stability: Unstable
+
+    @type connectionCount: C{int}
+    @ivar connectionCount: number of current connections.
+    @type connectionLimit: C{int} or C{None}
+    @cvar connectionLimit: maximum number of connections.
+    @type overflowProtocol: L{Protocol} or C{None}
+    @cvar overflowProtocol: Protocol to use for new connections when
+        connectionLimit is exceeded.  If C{None} (the default value), excess
+        connections will be closed immediately.
+    """
+    connectionCount = 0
+    connectionLimit = None
+    overflowProtocol = None
+
+    def buildProtocol(self, addr):
+        if (self.connectionLimit is None or
+            self.connectionCount < self.connectionLimit):
+                # Build the normal protocol
+                wrappedProtocol = self.protocol()
+        elif self.overflowProtocol is None:
+            # Just drop the connection
+            return None
+        else:
+            # Too many connections, so build the overflow protocol
+            wrappedProtocol = self.overflowProtocol()
+
+        wrappedProtocol.factory = self
+        protocol = ProtocolWrapper(self, wrappedProtocol)
+        self.connectionCount += 1
+        return protocol
+
+    def registerProtocol(self, p):
+        pass
+
+    def unregisterProtocol(self, p):
+        self.connectionCount -= 1
+
+
 class TimeoutProtocol(ProtocolWrapper):
     """Protocol that automatically disconnects when the connection is idle.
-    
+
     Stability: Unstable
     """
 
@@ -319,7 +356,7 @@ class TimeoutProtocol(ProtocolWrapper):
 
     def setTimeout(self, timeoutPeriod=None):
         """Set a timeout.
-        
+
         This will cancel any existing timeouts.
 
         @param timeoutPeriod: If not C{None}, change the timeout period.
@@ -332,7 +369,7 @@ class TimeoutProtocol(ProtocolWrapper):
 
     def cancelTimeout(self):
         """Cancel the timeout.
-        
+
         If the timeout was already cancelled, this does nothing.
         """
         if self.timeoutCall:
@@ -341,7 +378,7 @@ class TimeoutProtocol(ProtocolWrapper):
             except error.AlreadyCalled:
                 pass
             self.timeoutCall = None
-    
+
     def resetTimeout(self):
         """Reset the timeout, usually because some activity just happened."""
         if self.timeoutCall:
@@ -385,7 +422,74 @@ class TimeoutFactory(WrappingFactory):
 
     def buildProtocol(self, addr):
         return self.protocol(self, self.wrappedFactory.buildProtocol(addr),
-                             timeoutPeriod=self.timeoutPeriod)    
+                             timeoutPeriod=self.timeoutPeriod)
+
+
+class TrafficLoggingProtocol(ProtocolWrapper):
+    _counter = 0
+
+    def __init__(self, factory, wrappedProtocol, logfile, lengthLimit=None):
+        ProtocolWrapper.__init__(self, factory, wrappedProtocol)
+        self.logfile = logfile
+        self.lengthLimit = lengthLimit
+        TrafficLoggingProtocol._counter += 1
+        self._number = TrafficLoggingProtocol._counter
+
+    def _log(self, line):
+        self.logfile.write(line + '\n')
+        self.logfile.flush()
+
+    def _mungeData(self, data):
+        if self.lengthLimit and len(data) > self.lengthLimit:
+            data = data[:self.lengthLimit - 12] + '<... elided>'
+        return data
+
+    # IProtocol
+    def connectionMade(self):
+        self._log('*')
+        return ProtocolWrapper.connectionMade(self)
+
+    def dataReceived(self, data):
+        self._log('C %d: %r' % (self._number, self._mungeData(data)))
+        return ProtocolWrapper.dataReceived(self, data)
+
+    def connectionLost(self, reason):
+        self._log('C %d: %r' % (self._number, reason))
+        return ProtocolWrapper.connectionLost(self, reason)
+
+
+    # ITransport
+    def write(self, data):
+        self._log('S %d: %r' % (self._number, self._mungeData(data)))
+        return ProtocolWrapper.write(self, data)
+
+    def writeSequence(self, iovec):
+        self._log('SV %d: %r' % (self._number, [self._mungeData(d) for d in iovec]))
+        return ProtocolWrapper.writeSequence(self, iovec)
+
+    def loseConnection(self):
+        self._log('S %d: *' % (self._number,))
+        return ProtocolWrapper.loseConnection(self)
+
+
+class TrafficLoggingFactory(WrappingFactory):
+    protocol = TrafficLoggingProtocol
+
+    _counter = 0
+
+    def __init__(self, wrappedFactory, logfilePrefix, lengthLimit=None):
+        self.logfilePrefix = logfilePrefix
+        self.lengthLimit = lengthLimit
+        WrappingFactory.__init__(self, wrappedFactory)
+
+    def open(self, name):
+        return file(name, 'w')
+
+    def buildProtocol(self, addr):
+        self._counter += 1
+        logfile = self.open(self.logfilePrefix + '-' + str(self._counter))
+        return self.protocol(self, self.wrappedFactory.buildProtocol(addr),
+                             logfile, self.lengthLimit)
 
 
 class TimeoutMixin:
@@ -397,21 +501,25 @@ class TimeoutMixin:
 
     __timeoutCall = None
 
+    def callLater(self, period, func):
+        return reactor.callLater(period, func)
+
+
     def resetTimeout(self):
         """Reset the timeout count down"""
         if self.__timeoutCall is not None and self.timeOut is not None:
             self.__timeoutCall.reset(self.timeOut)
-    
+
     def setTimeout(self, period):
         """Change the timeout period
-        
+
         @type period: C{int} or C{NoneType}
         @param period: The period, in seconds, to change the timeout to, or
         C{None} to disable the timeout.
         """
         prev = self.timeOut
         self.timeOut = period
-        
+
         if self.__timeoutCall is not None:
             if period is None:
                 self.__timeoutCall.cancel()
@@ -419,8 +527,8 @@ class TimeoutMixin:
             else:
                 self.__timeoutCall.reset(period)
         elif period is not None:
-            self.__timeoutCall = reactor.callLater(period, self.__timedOut)
-            
+            self.__timeoutCall = self.callLater(period, self.__timedOut)
+
         return prev
 
     def __timedOut(self):
