@@ -7,6 +7,7 @@
 import os
 import sys
 import time
+import imp
 
 def shortPythonVersion():
     hv = sys.hexversion
@@ -62,8 +63,19 @@ class Platform:
                 return 1
             except WindowsError:
                 return 0
-        # not windows at all...
+        # not windows NT
         return 0
+    
+    def isWindows(self):
+        return self.getType() == 'win32'
+
+    def supportsThreads(self):
+        """Can threads be created?
+        """
+        try:
+            return imp.find_module('thread')[0] is None
+        except ImportError:
+            return False
 
 platform = Platform()
 platformType = platform.getType()
