@@ -14,7 +14,6 @@ instead of creating a thread pool directly.
 import Queue
 import threading
 import threadable
-import traceback
 import copy
 import sys
 
@@ -32,9 +31,9 @@ class ThreadPool:
     """
     This class (hopefully) generalizes the functionality of a pool of
     threads to which work can be dispatched.
-    
+
     dispatch(), dispatchWithCallback() and stop() should only be called from
-    a single thread, unless you make a subclass where stop() and 
+    a single thread, unless you make a subclass where stop() and
     _startSomeWorkers() are synchronized.
     """
     __inited = 0
@@ -43,8 +42,9 @@ class ThreadPool:
     joined = 0
     started = 0
     workers = 0
-    
-    def __init__(self, minthreads=5, maxthreads=20):
+    name = None
+
+    def __init__(self, minthreads=5, maxthreads=20, name=None):
         """Create a new threadpool.
 
         @param minthreads: minimum number of threads in the pool
@@ -56,6 +56,7 @@ class ThreadPool:
         self.q = Queue.Queue(0)
         self.min = minthreads
         self.max = maxthreads
+        self.name = name
         if runtime.platform.getType() != "java":
             self.waiters = []
             self.threads = []
@@ -75,7 +76,7 @@ class ThreadPool:
 
     def startAWorker(self):
         self.workers = self.workers + 1
-        name = "PoolThread-%s-%s" % (id(self), self.workers)
+        name = "PoolThread-%s-%s" % (self.name or id(self), self.workers)
         try:
             firstJob = self.q.get(0)
         except Queue.Empty:
@@ -97,12 +98,12 @@ class ThreadPool:
         state['min'] = self.min
         state['max'] = self.max
         return state
-    
+
     def _startSomeWorkers(self):
-        if self.workers >= self.max:
-            return
-        # FIXME: Wait for any waiters to eat of the queue.
-        while self.workers < self.max and self.q.qsize() > 0:
+        while (
+            self.workers < self.max and # Don't create too many
+            len(self.waiters) < self.q.qsize() # but create enough
+            ):
             self.startAWorker()
 
     def dispatch(self, owner, func, *args, **kw):
@@ -118,27 +119,29 @@ class ThreadPool:
         self.q.put(o)
         if self.started:
             self._startSomeWorkers()
-    
+
     def _runWithCallback(self, callback, errback, func, args, kwargs):
         try:
             result = apply(func, args, kwargs)
         except:
-            errback(sys.exc_value)
+            errback(sys.exc_info()[1])
         else:
             callback(result)
-    
+
     def dispatchWithCallback(self, owner, callback, errback, func, *args, **kw):
         """Dispatch a function, returning the result to a callback function.
-        
+
         The callback function will be called in the thread - make sure it is
-        thread-safe."""
+        thread-safe.
+        """
         self.callInThread(self._runWithCallback, callback, errback, func, args, kw)
 
     def _worker(self, o):
         ct = threading.currentThread()
         while 1:
-            if o is not None:
-                if o == WorkerStop: break
+            if o is WorkerStop:
+                break
+            elif o is not None:
                 self.working.append(ct)
                 ctx, function, args, kwargs = o
                 try:
@@ -152,7 +155,7 @@ class ThreadPool:
             self.waiters.remove(ct)
 
         self.threads.remove(ct)
-    
+
     def stop(self):
         """Shutdown the threads in the threadpool."""
         self.joined = 1
@@ -179,7 +182,7 @@ class ThreadPool:
         self.max = maxthreads
         if not self.started:
             return
-        
+
         # Kill of some threads if we have too many.
         while self.workers > self.max:
             self.stopAWorker()
@@ -188,7 +191,7 @@ class ThreadPool:
             self.startAWorker()
         # Start some threads if there is a need.
         self._startSomeWorkers()
-   
+
     def dumpStats(self):
         log.msg('queue: %s'   % self.q.queue)
         log.msg('waiters: %s' % self.waiters)
