@@ -4,11 +4,10 @@
 import sys, os
 
 from zope.interface import Interface, Attribute, implements
-from twisted.python.components import backwardsCompatImplements
 
 from twisted.application import service, compat, app
 from twisted.persisted import sob
-from twisted.python import log, usage, util, plugin as oldplugin
+from twisted.python import usage, util, plugin as oldplugin
 from twisted import plugin as newplugin
 
 class IServiceMaker(Interface):
@@ -71,7 +70,7 @@ def loadPlugins(debug = None, progress = None):
             shortTapName = plug.module.split('.')[-1]
         tapLookup[shortTapName] = plug
 
-    plugins = newplugin.getPlugIns(IServiceMaker)
+    plugins = newplugin.getPlugins(IServiceMaker)
     for plug in plugins:
         tapLookup[plug.tapname] = plug
 
@@ -234,4 +233,3 @@ class _tapHelper(object):
             return namedAny(self.module).makeService
         return get,
     makeService = property(*makeService())
-backwardsCompatImplements(_tapHelper)
