@@ -1,19 +1,7 @@
 
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """
 This module represents flavors of remotely acessible objects.
@@ -22,81 +10,61 @@ Currently this is only objects accessible through Perspective Broker, but will
 hopefully encompass all forms of remote access which can emulate subsets of PB
 (such as XMLRPC or SOAP).
 
+Stability: semi-stable
+
+Future Plans: Optimization.  Exploitation of new-style object model.
+Optimizations to this module should not affect external-use semantics at all,
+but may have a small impact on users who subclass and override methods.
+
+@author: U{Glyph Lefkowitz<mailto:glyph@twistedmatrix.com>}
 """
 
+__version__ = "$Revision: 1.32 $"[11:-2]
+
+# NOTE: this module should NOT import pb; it is supposed to be a module which
+# abstractly defines remotely accessible types.  Many of these types expect to
+# be serialized by Jelly, but they ought to be accessible through other
+# mechanisms (like XMLRPC)
+
 # system imports
 import types
+from zope.interface import implements
 
 # twisted imports
-from twisted.python import log
+from twisted.python import log, reflect, components
 
+# sibling imports
+from jelly import setUnjellyableForClass, setUnjellyableForClassTree, setUnjellyableFactoryForClass, unjellyableRegistry
+from jelly import Jellyable, Unjellyable, _Dummy
+from jelly import setInstanceState, getInstanceState
+
+# compatibility
+setCopierForClass = setUnjellyableForClass
+setCopierForClassTree = setUnjellyableForClassTree
+setFactoryForClass = setUnjellyableFactoryForClass
+copyTags = unjellyableRegistry
 
 copy_atom = "copy"
 cache_atom = "cache"
 cached_atom = "cached"
 remote_atom = "remote"
 
-copyTags = {}
 
-def setCopierForClass(classname, copier):
-    """Set which local class will represent a remote type.
-
-    If you have written a Copyable class that you expect your client to be
-    receiving, write a local "copy" class to represent it, then call::
-
-      pb.setCopierForClass('module.package.Class', MyCopier).
-
-    Call this at the module level immediately after its class
-    definition. MyCopier should be a subclass of RemoteCopy.
-
-    The classname may be a special tag returned by
-    'Copyable.getTypeToCopyFor' rather than an actual classname.
-
-    This call is also for cached classes, since there will be no
-    overlap.  The rules are the same.
-    """
-
-    global copyTags
-    copyTags[classname] = copier
-
-def setCopierForClassTree(module, baseClass, prefix=None):
-    """Set all classes in a module derived from baseClass as copiers for a corresponding remote class.
-
-    When you have a heirarchy of Copyable (or Cacheable) classes on
-    one side, and a mirror structure of Copied (or RemoteCache)
-    classes on the other, use this to setCopierForClass all your
-    Copieds for the Copyables.
-
-    Each copyTag (the \"classname\" argument to getTypeToCopyFor, and
-    what the Copyable's getTypeToCopyFor returns) is formed from
-    adding a prefix to the Copied's class name.  The prefix defaults
-    to module.__name__.  If you wish the copy tag to consist of solely
-    the classname, pass the empty string \'\'.
-
-    module -- a module object from which to pull the Copied classes.
-              (passing sys.modules[__name__] might be useful)
-
-    baseClass -- the base class from which all your Copied classes derive.
-
-    prefix -- the string prefixed to classnames to form the copyTags.
-    """
-    if prefix is None:
-        prefix = module.__name__
-
-    if prefix:
-        prefix = "%s." % prefix
-
-    for i in dir(module):
-        i_ = getattr(module, i)
-        if type(i_) == types.ClassType:
-            if issubclass(i_, baseClass):
-                setCopierForClass('%s%s' % (prefix, i), i_)
+class NoSuchMethod(AttributeError):
+    """Raised if there is no such remote method"""
 
 
-class Serializable:
-    """(internal) An object that can be passed remotely.
+class IPBRoot(components.Interface):
+    """Factory for root Referenceable objects for PB servers."""
 
-    This is a style of object which can be serialized by Perspective
+    def rootObject(self, broker):
+        """Return root Referenceable for broker."""
+
+
+class Serializable(Jellyable):
+    """An object that can be passed remotely.
+
+    I am a style of object which can be serialized by Perspective
     Broker.  Objects which wish to be referenceable or copied remotely
     have to subclass Serializable.  However, clients of Perspective
     Broker will probably not want to directly subclass Serializable; the
@@ -106,19 +74,13 @@ class Serializable:
     passed to or returned from a remote method.  Certain basic types
     (dictionaries, lists, tuples, numbers, strings) are serializable by
     default; however, classes need to choose a specific serialization
-    style: Referenceable, Viewable, Copyable or Cacheable.
+    style: L{Referenceable}, L{Viewable}, L{Copyable} or L{Cacheable}.
 
-    You may also pass [lists, dictionaries, tuples] of Serializable
+    You may also pass C{[lists, dictionaries, tuples]} of L{Serializable}
     instances to or return them from remote methods, as many levels deep
     as you like.
     """
 
-    def remoteSerialize(self, broker):
-        """Return a list of strings & numbers which represents this object remotely.
-        """
-
-        raise NotImplementedError()
-
     def processUniqueID(self):
         """Return an ID which uniquely represents this object for this process.
 
@@ -150,59 +112,66 @@ class Referenceable(Serializable):
         """
         args = broker.unserialize(args)
         kw = broker.unserialize(kw)
-        method = getattr(self, "remote_%s" % message)
+        method = getattr(self, "remote_%s" % message, None)
+        if method is None:
+            raise NoSuchMethod("No such method: remote_%s" % (message,))
         try:
-            state = apply(method, args, kw)
+            state = method(*args, **kw)
         except TypeError:
-            print ("%s didn't accept %s and %s" % (method, args, kw))
+            log.msg("%s didn't accept %s and %s" % (method, args, kw))
             raise
         return broker.serialize(state, self.perspective)
 
-    def remoteSerialize(self, broker):
+    def jellyFor(self, jellier):
         """(internal)
 
         Return a tuple which will be used as the s-expression to
         serialize this to a peer.
         """
 
-        return remote_atom, broker.registerReference(self)
+        return "remote", jellier.invoker.registerReference(self)
 
 
 class Root(Referenceable):
-    """I provide a root object to Brokers for a BrokerFactory.
+    """I provide a root object to L{pb.Broker}s for a L{pb.BrokerFactory}.
 
-    When a BrokerFactory produces a Broker, it supplies that Broker
-    with an object named "root".  That object is obtained by calling
-    my rootObject method.
+    When a L{pb.BrokerFactory} produces a L{pb.Broker}, it supplies that
+    L{pb.Broker} with an object named \"root\".  That object is obtained
+    by calling my rootObject method.
 
-    See also: getObjectAt
+    See also: L{pb.getObjectAt}
     """
 
+    implements(IPBRoot)
+    
     def rootObject(self, broker):
-        """A BrokerFactory is requesting to publish me as a root object.
+        """A L{pb.BrokerFactory} is requesting to publish me as a root object.
 
-        When a BrokerFactory is sending me as the root object, this
+        When a L{pb.BrokerFactory} is sending me as the root object, this
         method will be invoked to allow per-broker versions of an
         object.  By default I return myself.
         """
         return self
 
+
 class ViewPoint(Referenceable):
-    """I act as an indirect reference to an object accessed through a Perspective.
+    """
+    I act as an indirect reference to an object accessed through a
+    L{pb.Perspective}.
 
     Simply put, I combine an object with a perspective so that when a
     peer calls methods on the object I refer to, the method will be
     invoked with that perspective as a first argument, so that it can
     know who is calling it.
 
-    While Viewable objects will be converted to ViewPoints by default
+    While L{Viewable} objects will be converted to ViewPoints by default
     when they are returned from or sent as arguments to a remote
     method, any object may be manually proxied as well. (XXX: Now that
-    this class is no longer named Proxy, this is the only occourance
+    this class is no longer named C{Proxy}, this is the only occourance
     of the term 'proxied' in this docstring, and may be unclear.)
 
-    This can be useful when dealing with Perspectives, Copyables,
-    and Cacheables.  It is legal to implement a method as such on
+    This can be useful when dealing with L{pb.Perspective}s, L{Copyable}s,
+    and L{Cacheable}s.  It is legal to implement a method as such on
     a perspective::
 
      | def perspective_getViewPointForOther(self, name):
@@ -212,16 +181,16 @@ class ViewPoint(Referenceable):
 
     This will allow you to have references to Perspective objects in two
     different ways.  One is through the initial 'attach' call -- each
-    peer will have a RemoteReference to their perspective directly.  The
-    other is through this method; each peer can get a RemoteReference to
-    all other perspectives in the service; but that RemoteReference will
-    be to a ViewPoint, not directly to the object.
+    peer will have a L{pb.RemoteReference} to their perspective directly.  The
+    other is through this method; each peer can get a L{pb.RemoteReference} to
+    all other perspectives in the service; but that L{pb.RemoteReference} will
+    be to a L{ViewPoint}, not directly to the object.
 
     The practical offshoot of this is that you can implement 2 varieties
     of remotely callable methods on this Perspective; view_xxx and
-    perspective_xxx. view_xxx methods will follow the rules for
-    ViewPoint methods (see ViewPoint.remoteMessageReceived), and
-    perspective_xxx methods will follow the rules for Perspective
+    C{perspective_xxx}. C{view_xxx} methods will follow the rules for
+    ViewPoint methods (see ViewPoint.L{remoteMessageReceived}), and
+    C{perspective_xxx} methods will follow the rules for Perspective
     methods.
     """
 
@@ -236,17 +205,11 @@ class ViewPoint(Referenceable):
         """
         return (id(self.perspective), id(self.object))
 
-    def remoteSerialize(self, broker):
-        """(internal) Serialize remotely.
-        """
-        # waste not, want not
-        return remote_atom, broker.registerReference(self)
-
     def remoteMessageReceived(self, broker, message, args, kw):
         """A remote message has been received.  Dispatch it appropriately.
 
         The default implementation is to dispatch to a method called
-        'view_messagename' to my Object and call it on my object with
+        'C{view_messagename}' to my Object and call it on my object with
         the same arguments, modified by inserting my Perspective as
         the first argument.
         """
@@ -256,25 +219,25 @@ class ViewPoint(Referenceable):
         try:
             state = apply(method, (self.perspective,)+args, kw)
         except TypeError:
-            print ("%s didn't accept %s and %s" % (method, args, kw))
+            log.msg("%s didn't accept %s and %s" % (method, args, kw))
             raise
         rv = broker.serialize(state, self.perspective, method, args, kw)
         return rv
 
 
 class Viewable(Serializable):
-    """I will be converted to a ViewPoint when passed to or returned from a remote method.
+    """I will be converted to a L{ViewPoint} when passed to or returned from a remote method.
 
     The beginning of a peer's interaction with a PB Service is always
-    through a perspective.  However, if a perspective_xxx method returns
+    through a perspective.  However, if a C{perspective_xxx} method returns
     a Viewable, it will be serialized to the peer as a response to that
     method.
     """
 
-    def remoteSerialize(self, broker):
-        """Serialize a ViewPoint for me and the perspective of the given broker.
+    def jellyFor(self, jellier):
+        """Serialize a L{ViewPoint} for me and the perspective of the given broker.
         """
-        return ViewPoint(broker.serializingPerspective, self).remoteSerialize(broker)
+        return ViewPoint(jellier.invoker.serializingPerspective, self).jellyFor(jellier)
 
 
 
@@ -287,7 +250,7 @@ class Copyable(Serializable):
     the peer.
 
     The peer will then look up the type to represent this with; see
-    RemoteCopy for details.
+    L{RemoteCopy} for details.
     """
 
     def getStateToCopy(self):
@@ -300,9 +263,11 @@ class Copyable(Serializable):
         return self.__dict__
 
     def getStateToCopyFor(self, perspective):
-        """Gather state to send when I am serialized for a particular perspective.
+        """
+        Gather state to send when I am serialized for a particular
+        perspective.
 
-        I will default to calling getStateToCopy.  Override this to
+        I will default to calling L{getStateToCopy}.  Override this to
         customize this behavior.
         """
 
@@ -316,28 +281,32 @@ class Copyable(Serializable):
         you may override this to change it.
         """
 
-        return str(self.__class__)
+        return reflect.qual(self.__class__)
 
     def getTypeToCopyFor(self, perspective):
         """Determine what type tag to send for me.
 
-        By default, defer to self.getTypeToCopy() normally this is
+        By default, defer to self.L{getTypeToCopy}() normally this is
         adequate, but you may override this to change it.
         """
 
         return self.getTypeToCopy()
 
-    def remoteSerialize(self, broker):
+    def jellyFor(self, jellier):
         """Assemble type tag and state to copy for this broker.
 
-        This will call getTypeToCopyFor and getStateToCopy, and
-        return an appropriate s-expression to represent me.  Do
-        not override this method.
+        This will call L{getTypeToCopyFor} and L{getStateToCopy}, and
+        return an appropriate s-expression to represent me.
         """
 
-        p = broker.serializingPerspective
-        return (copy_atom, self.getTypeToCopyFor(p),
-                broker.jelly(self.getStateToCopyFor(p)))
+        if jellier.invoker is None:
+            return getInstanceState(self, jellier)
+        p = jellier.invoker.serializingPerspective
+        t = self.getTypeToCopyFor(p)
+        state = self.getStateToCopyFor(p)
+        sxp = jellier.prepare(self)
+        sxp.extend([t, jellier.jelly(state)])
+        return jellier.preserve(self, sxp)
 
 
 class Cacheable(Copyable):
@@ -353,57 +322,65 @@ class Cacheable(Copyable):
     """
 
     def getStateToCacheAndObserveFor(self, perspective, observer):
-        """Get state to cache on the client and client-cache reference to observe locally.
+        """
+        Get state to cache on the client and client-cache reference
+        to observe locally.
 
         This is similiar to getStateToCopyFor, but it additionally
         passes in a reference to the client-side RemoteCache instance
         that will be created when it is unserialized.  This allows
         Cacheable instances to keep their RemoteCaches up to date when
-        they change, such that no changes can occurr between the point
+        they change, such that no changes can occur between the point
         at which the state is initially copied and the client receives
         it that are not propogated.
         """
 
         return self.getStateToCopyFor(perspective)
 
-    def remoteSerialize(self, broker):
+    def jellyFor(self, jellier):
         """Return an appropriate tuple to serialize me.
 
         Depending on whether this broker has cached me or not, this may
         return either a full state or a reference to an existing cache.
         """
-
-        luid = broker.cachedRemotelyAs(self)
+        if jellier.invoker is None:
+            return getInstanceState(self, jellier)
+        luid = jellier.invoker.cachedRemotelyAs(self, 1)
         if luid is None:
-            luid = broker.cacheRemotely(self)
-            p = broker.serializingPerspective
+            luid = jellier.invoker.cacheRemotely(self)
+            p = jellier.invoker.serializingPerspective
             type_ = self.getTypeToCopyFor(p)
-            observer = RemoteCacheObserver(broker, self, p)
+            observer = RemoteCacheObserver(jellier.invoker, self, p)
             state = self.getStateToCacheAndObserveFor(p, observer)
-            jstate = broker.jelly(state)
-            return cache_atom, luid, type_, jstate
+            l = jellier.prepare(self)
+            jstate = jellier.jelly(state)
+            l.extend([type_, luid, jstate])
+            return jellier.preserve(self, l)
         else:
             return cached_atom, luid
 
+    def stoppedObserving(self, perspective, observer):
+        """This method is called when a client has stopped observing me.
+
+        The 'observer' argument is the same as that passed in to
+        getStateToCacheAndObserveFor.
+        """
 
 
-class RemoteCopy:
+
+class RemoteCopy(Unjellyable):
     """I am a remote copy of a Copyable object.
 
-    When the state from a Copyable object is received, an instance will
-    be created based on the copy tags table (see setCopierForClass) and
-    sent the setCopyableState message.  I provide a reasonable default
+    When the state from a L{Copyable} object is received, an instance will
+    be created based on the copy tags table (see setUnjellyableForClass) and
+    sent the L{setCopyableState} message.  I provide a reasonable default
     implementation of that message; subclass me if you wish to serve as
     a copier for remote data.
 
     NOTE: copiers are invoked with no arguments.  Do not implement a
-    constructor which requires args in a subclass of RemoteCopy!
+    constructor which requires args in a subclass of L{RemoteCopy}!
     """
 
-    def postUnjelly(self):
-        """I will be invoked after the data I was sent with has been fully unjellied.
-        """
-
     def setCopyableState(self, state):
         """I will be invoked with the state to copy locally.
 
@@ -415,13 +392,20 @@ class RemoteCopy:
 
         self.__dict__ = state
 
+    def unjellyFor(self, unjellier, jellyList):
+        if unjellier.invoker is None:
+            return setInstanceState(self, unjellier, jellyList)
+        self.setCopyableState(unjellier.unjelly(jellyList[1]))
+        return self
+
+
 
 class RemoteCache(RemoteCopy, Serializable):
-    """A cache is a local representation of a remote Cacheable object.
+    """A cache is a local representation of a remote L{Cacheable} object.
 
     This represents the last known state of this object.  It may
     also have methods invoked on it -- in order to update caches,
-    the cached class generates a RemoteReference to this object as
+    the cached class generates a L{pb.RemoteReference} to this object as
     it is originally sent.
 
     Much like copy, I will be invoked with no arguments.  Do not
@@ -433,7 +417,7 @@ class RemoteCache(RemoteCopy, Serializable):
         """A remote message has been received.  Dispatch it appropriately.
 
         The default implementation is to dispatch to a method called
-        'observe_messagename' and call it on my  with the same arguments.
+        'C{observe_messagename}' and call it on my  with the same arguments.
         """
 
         args = broker.unserialize(args)
@@ -442,24 +426,48 @@ class RemoteCache(RemoteCopy, Serializable):
         try:
             state = apply(method, args, kw)
         except TypeError:
-            print ("%s didn't accept %s and %s" % (method, args, kw))
+            log.msg("%s didn't accept %s and %s" % (method, args, kw))
             raise
         return broker.serialize(state, None, method, args, kw)
 
-    def remoteSerialize(self, broker):
+    def jellyFor(self, jellier):
         """serialize me (only for the broker I'm for) as the original cached reference
         """
-
-        assert broker is self.broker, "You cannot exchange cached proxies between brokers."
-        print 'local cache',self.luid
+        if jellier.invoker is None:
+            return getInstanceState(self, jellier)
+        assert jellier.invoker is self.broker, "You cannot exchange cached proxies between brokers."
         return 'lcache', self.luid
 
-    def __really_del__(self):
-        """Final finalization call, made after all remote references have been lost.
-        """
+
+    def unjellyFor(self, unjellier, jellyList):
+        if unjellier.invoker is None:
+            return setInstanceState(self, unjellier, jellyList)
+        self.broker = unjellier.invoker
+        self.luid = jellyList[1]
+        cProxy = _Dummy()
+        cProxy.__class__ = self.__class__
+        cProxy.__dict__ = self.__dict__
+        # XXX questionable whether this was a good design idea...
+        init = getattr(cProxy, "__init__", None)
+        if init:
+            init()
+        unjellier.invoker.cacheLocally(jellyList[1], self)
+        cProxy.setCopyableState(unjellier.unjelly(jellyList[2]))
+        # Might have changed due to setCopyableState method; we'll assume that
+        # it's bad form to do so afterwards.
+        self.__dict__ = cProxy.__dict__
+        # chomp, chomp -- some existing code uses "self.__dict__ =", some uses
+        # "__dict__.update".  This is here in order to handle both cases.
+        self.broker = unjellier.invoker
+        self.luid = jellyList[1]
+        return cProxy
+
+##     def __really_del__(self):
+##         """Final finalization call, made after all remote references have been lost.
+##         """
 
     def __cmp__(self, other):
-        """Compare me [to another RemoteCacheProxy.
+        """Compare me [to another RemoteCache.
         """
         if isinstance(other, self.__class__):
             return cmp(id(self.__dict__), id(other.__dict__))
@@ -478,14 +486,38 @@ class RemoteCache(RemoteCopy, Serializable):
         """Do distributed reference counting on finalize.
         """
         try:
-            # print 'decache: %s %d' % (self, self.luid)
+            # log.msg( ' --- decache: %s %s' % (self, self.luid) )
             if self.broker:
                 self.broker.decCacheRef(self.luid)
         except:
             log.deferr()
 
+def unjellyCached(unjellier, unjellyList):
+    luid = unjellyList[1]
+    cNotProxy = unjellier.invoker.cachedLocallyAs(luid)
+
+    cProxy = _Dummy()
+    cProxy.__class__ = cNotProxy.__class__
+    cProxy.__dict__ = cNotProxy.__dict__
+    return cProxy
+
+setUnjellyableForClass("cached", unjellyCached)
+
+def unjellyLCache(unjellier, unjellyList):
+    luid = unjellyList[1]
+    obj = unjellier.invoker.remotelyCachedForLUID(luid)
+    return obj
+
+setUnjellyableForClass("lcache", unjellyLCache)
+
+def unjellyLocal(unjellier, unjellyList):
+    obj = unjellier.invoker.localObjectForID(unjellyList[1])
+    return obj
+
+setUnjellyableForClass("local", unjellyLocal)
+
 class RemoteCacheMethod:
-    """A method on a reference to a RemoteCache.
+    """A method on a reference to a L{RemoteCache}.
     """
 
     def __init__(self, name, broker, cached, perspective):
@@ -498,10 +530,10 @@ class RemoteCacheMethod:
 
     def __cmp__(self, other):
         return cmp((self.name, self.broker, self.perspective, self.cached), other)
-    
+
     def __hash__(self):
         return hash((self.name, self.broker, self.perspective, self.cached))
-    
+
     def __call__(self, *args, **kw):
         """(internal) action method.
         """
@@ -512,16 +544,23 @@ class RemoteCacheMethod:
         return self.broker._sendMessage('cache', self.perspective, cacheID, self.name, args, kw)
 
 class RemoteCacheObserver:
-    """I am a reverse-reference to the peer's RemoteCache.
+    """I am a reverse-reference to the peer's L{RemoteCache}.
 
     I am generated automatically when a cache is serialized.  I
-    represent a reference to the client's RemoteCache object that
-    will represent a particular Cacheable; I am the additional
+    represent a reference to the client's L{RemoteCache} object that
+    will represent a particular L{Cacheable}; I am the additional
     object passed to getStateToCacheAndObserveFor.
     """
 
     def __init__(self, broker, cached, perspective):
-        """Initialize me pointing at a client side cache for a particular broker/perspective.
+        """(internal) Initialize me.
+
+        @param broker: a L{pb.Broker} instance.
+
+        @param cached: a L{Cacheable} instance that this L{RemoteCacheObserver}
+            corresponds to.
+
+        @param perspective: a reference to the perspective who is observing this.
         """
 
         self.broker = broker
@@ -533,7 +572,7 @@ class RemoteCacheObserver:
             self.broker, self.cached, self.perspective, id(self))
 
     def __hash__(self):
-        """generate a hash unique to all RemoteCacheObservers for this broker/perspective/cached triplet
+        """Generate a hash unique to all L{RemoteCacheObserver}s for this broker/perspective/cached triplet
         """
 
         return (  (hash(self.broker) % 2**10)
@@ -541,31 +580,23 @@ class RemoteCacheObserver:
                 + (hash(self.cached) % 2**10))
 
     def __cmp__(self, other):
-        """compare me to another RemoteCacheObserver
+        """Compare me to another L{RemoteCacheObserver}.
         """
 
         return cmp((self.broker, self.perspective, self.cached), other)
 
-    def callRemote(self, name, *args, **kw):
+    def callRemote(self, _name, *args, **kw):
         """(internal) action method.
         """
         cacheID = self.broker.cachedRemotelyAs(self.cached)
         if cacheID is None:
             from pb import ProtocolError
-            raise ProtocolError("You can't call a cached method when the object hasn't been given to the peer yet.")
-        return self.broker._sendMessage('cache', self.perspective, cacheID, name, args, kw)
-
-    def __getattr__(self, key):
-        """Create a RemoteCacheMethod.
-        """
-        if key[:2]=='__' and key[-2:]=='__':
-            raise AttributeError(key)
-        import traceback
-        file, lineno, func, nne = traceback.extract_stack()[-2] # caller
-        log.msg("%s:%s %s calls obsolete 'transparent' RemoteCacheMethod %s" % (file, lineno, func, key))
-        return self.remoteMethod(key)
+            raise ProtocolError("You can't call a cached method when the "
+                                "object hasn't been given to the peer yet.")
+        return self.broker._sendMessage('cache', self.perspective, cacheID,
+                                        _name, args, kw)
 
     def remoteMethod(self, key):
-        """Get a RemoteMethod for this key.
+        """Get a L{pb.RemoteMethod} for this key.
         """
         return RemoteCacheMethod(key, self.broker, self.cached, self.perspective)
